package it.unibas.mediapesata.modello;

import java.io.IOException;

public class Studente {

    private String nome;
    private String cognome;
    private int matricola;
    private java.util.ArrayList listaEsami = new java.util.ArrayList();
    
    public Studente() {}
    
    public Studente (String nome, String cognome, int matricola) {
        this.nome = nome;
        this.cognome = cognome;
        this.matricola = matricola;
    }
    
    public String getNome() {
        return this.nome;
    }

    public String getCognome() {
        return this.cognome;
    }

    public int getMatricola() {
        return this.matricola;
    }

    public void addEsame(String insegnamento, int voto, boolean lode, int crediti) {
        Esame esame = new Esame(insegnamento, voto, lode, crediti);
        this.listaEsami.add(esame);
    }

    public void addEsame(Esame esame) {
        this.listaEsami.add(esame);
    }

    public Esame getEsame(int i) {
        if (i < 0 || i >= this.listaEsami.size()) {
            throw new IndexOutOfBoundsException("Esame inesistente");
        }
        return (Esame)this.listaEsami.get(i);
    }

    public int getNumeroEsami() {
        return this.listaEsami.size();
    }

    public double getMediaPesata() {
        if (this.listaEsami.size() == 0) {
            throw new IllegalArgumentException("Non e' possibile calcolare la media di 0 esami");
        }
        int sommaVotiPesati = 0;
        int sommaCrediti = 0;
        for (int i = 0; i < this.listaEsami.size(); i++) {
            Esame esameIesimo = (Esame)this.listaEsami.get(i);
            sommaVotiPesati += esameIesimo.getVoto() * esameIesimo.getCrediti();
            sommaCrediti += esameIesimo.getCrediti();
        }
        return ((double)sommaVotiPesati)/sommaCrediti;
    }

    public String toString() {
        return "Cognome: " + getCognome() + " - Nome: " + getNome()
             + " - Matricola: " + getMatricola();
    }
    
    public String toSaveString() {
        return getCognome() + " | " + getNome() + " | " + getMatricola();
    }

    /* ******************************************
     *               Salvataggio
     * ****************************************** */

    public void salva(String nomeFile) throws PersistenceException {
        java.io.PrintWriter flusso = null;
        try {
            java.io.FileWriter fileWriter = new java.io.FileWriter(nomeFile);
            flusso = new java.io.PrintWriter(fileWriter);
            flusso.println("Studente:");
            flusso.println(this.toSaveString());
            flusso.println("--------------------------------------");
            salvaEsami(flusso);
        } catch (IOException ioe) {
            throw new PersistenceException(ioe);
        } finally {
            if (flusso != null) {
                flusso.close();
            }
        }
    }
    
    private void salvaEsami(java.io.PrintWriter flusso) {
        for (int i = 0; i < getNumeroEsami(); i++) {
            flusso.println(getEsame(i).toSaveString());            
        }
    }

    /* ******************************************
     *               Caricamento
     * ****************************************** */

    public static Studente carica(String nomeFile) throws PersistenceException {
        Studente studente = new Studente();
        java.io.BufferedReader flusso = null;
        try {
            java.io.FileReader fileReader = new java.io.FileReader(nomeFile);
            flusso = new java.io.BufferedReader(fileReader);
            estraiDatiStudente(studente, flusso);
            caricaEsami(studente, flusso);
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            try {
                if (flusso != null) {
                    flusso.close();
                }
            } catch (java.io.IOException ioe) {}
        }
        return studente;
    }
        
    private static void estraiDatiStudente(Studente studente, java.io.BufferedReader flusso) 
                   throws java.io.IOException {
        flusso.readLine();
        String lineaStudente = flusso.readLine();
        java.util.StringTokenizer tokenizer = new java.util.StringTokenizer(lineaStudente, "|");
        studente.cognome = tokenizer.nextToken().trim();
        studente.nome = tokenizer.nextToken().trim();
        studente.matricola = Integer.parseInt(tokenizer.nextToken().trim());
    }
    
    private static void caricaEsami(Studente studente, java.io.BufferedReader flusso) 
                   throws java.io.IOException {
        flusso.readLine();
        String lineaEsame;
        while ((lineaEsame = flusso.readLine()) != null) {
            studente.addEsame(estraiDatiEsame(lineaEsame, flusso));
        }
    }
    
    private static Esame estraiDatiEsame(String lineaEsame, java.io.BufferedReader flusso) {
        java.util.StringTokenizer tokenizer = new java.util.StringTokenizer(lineaEsame, "|");
        String insegnamento = tokenizer.nextToken().trim();
        int crediti = Integer.parseInt(tokenizer.nextToken().trim());
        int voto = Integer.parseInt(tokenizer.nextToken().trim());
        boolean lode = Boolean.valueOf(tokenizer.nextToken().trim());
        return new Esame(insegnamento, voto, lode, crediti);
    }
    

}


