package it.unibas.mediapesata.test;

import junit.framework.*;

import it.unibas.mediapesata.modello.Studente;
import it.unibas.mediapesata.modello.Esame;

public class TestStudente extends TestCase {
        
    private static final String PREFISSO = "e:\\codice\\lavoro\\java\\mediaPesata\\test\\dati\\";
    private static final String NOMEFILE1 = PREFISSO + "studente1-test.txt";
    private static final String NOMEFILE2 = PREFISSO + "studente2-test.txt";
    private static final String NOMEFILE3 = PREFISSO + "studente3-test.txt";
    private static final String NOMEFILE4 = PREFISSO + "studente4-test.txt";

    private Studente studente;
    private Esame esame1;
    private Esame esame2;
    private Esame esame3;
    
    public void setUp() {
        //System.out.println("Sto eseguendo setUp()");
        studente = new Studente("Mario", "Rossi", 1234);
        esame1 = new Esame("Analisi", 24, false, 3);
        esame2 = new Esame("Geometria", 27, false, 9);
        esame3 = new Esame("Fisica", 30, true, 9);
    }
    
    public void testMediaPesata1() {
        //System.out.println("Sto eseguendo testMediaPesata1()");
        studente.addEsame(esame1);
        studente.addEsame(esame2);
        Assert.assertTrue("media p. 1", studente.getMediaPesata() == 26.25);
    }

    public void testMediaPesata2() {
        //System.out.println("Sto eseguendo testMediaPesata2()");
        studente.addEsame(esame1);
        Assert.assertTrue("media p. 2", studente.getMediaPesata() == 24);
    }
    
    public void testMediaPesata3() {
        //System.out.println("Sto eseguendo testMediaPesata3()");
        try {
            studente.getMediaPesata();
            Assert.fail();
        } catch (Exception e) { }
    }
    
    public void testCarica1() throws Exception {
        studente = Studente.carica(NOMEFILE1);
        verificaStudente1(studente);
    }
        
    public void testCarica2() throws Exception {
        studente = Studente.carica(NOMEFILE2);
        verificaStudente2(studente);
    }

    public void testSalva1() throws Exception {
        studente.addEsame(esame1);
        studente.addEsame(esame3);
        studente.salva(NOMEFILE3);
        Studente studente1 = Studente.carica(NOMEFILE3);
        verificaStudente1(studente1);
    }

    public void testSalva2() throws Exception {
        studente.salva(NOMEFILE4);
        Studente studente2 = Studente.carica(NOMEFILE4);
        verificaStudente2(studente2);
    }

    private void verificaStudente1(Studente studente) {
        assertEquals("Rossi", studente.getCognome());
        assertEquals("Mario", studente.getNome());
        assertEquals(1234, studente.getMatricola());
        assertEquals(2, studente.getNumeroEsami());
        assertEquals("Analisi", studente.getEsame(0).getInsegnamento());
        assertEquals(3, studente.getEsame(0).getCrediti());
        assertEquals(24, studente.getEsame(0).getVoto());
        assertEquals(false, studente.getEsame(0).isLode());
        assertEquals("Fisica", studente.getEsame(1).getInsegnamento());
        assertEquals(9, studente.getEsame(1).getCrediti());
        assertEquals(30, studente.getEsame(1).getVoto());
        assertEquals(true, studente.getEsame(1).isLode());        
    }
    
    private void verificaStudente2(Studente studente) {
        assertEquals("Rossi", studente.getCognome());
        assertEquals("Mario", studente.getNome());
        assertEquals(1234, studente.getMatricola());
        assertEquals(0, studente.getNumeroEsami());        
    }
}
